"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildUpdateDatastoreQuery = exports.onSubmitValidationRun = exports.buildDataStoreExpression = exports.replaceEmptyStringStatement = void 0;
/*
  Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.

  Licensed under the Apache License, Version 2.0 (the "License").
  You may not use this file except in compliance with the License.
  You may obtain a copy of the License at

      http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
const codegen_ui_1 = require("@aws-amplify/codegen-ui");
const typescript_1 = require("typescript");
const helpers_1 = require("../../helpers");
const model_values_1 = require("./model-values");
const relationship_1 = require("./relationship");
const map_from_fieldConfigs_1 = require("./map-from-fieldConfigs");
const bidirectional_relationship_1 = require("./bidirectional-relationship");
const parse_fields_1 = require("./parse-fields");
const getRecordUpdateDataStoreCallExpression = ({ modelName, importedModelName, fieldConfigs, }) => {
    const updatedObjectName = 'updated';
    const modelFieldsObjectName = 'modelFields';
    // TODO: remove after DataStore addresses issue: https://github.com/aws-amplify/amplify-js/issues/10750
    // temporary solution to remove hasOne & belongsTo records
    const relationshipBasedUpdates = (0, relationship_1.getRelationshipBasedRecordUpdateStatements)({
        updatedObjectName,
        modelFieldsObjectName,
        fieldConfigs,
    });
    return typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('DataStore'), typescript_1.factory.createIdentifier('save')), undefined, [
        typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier(importedModelName), typescript_1.factory.createIdentifier('copyOf')), undefined, [
            typescript_1.factory.createIdentifier(`${(0, helpers_1.lowerCaseFirst)(modelName)}Record`),
            typescript_1.factory.createArrowFunction(undefined, undefined, [
                typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createIdentifier(updatedObjectName), undefined, undefined, undefined),
            ], undefined, typescript_1.factory.createToken(typescript_1.SyntaxKind.EqualsGreaterThanToken), typescript_1.factory.createBlock([
                typescript_1.factory.createExpressionStatement(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('Object'), typescript_1.factory.createIdentifier('assign')), undefined, [
                    typescript_1.factory.createIdentifier(updatedObjectName),
                    (0, parse_fields_1.generateUpdateModelObject)(fieldConfigs, modelFieldsObjectName),
                ])),
                ...relationshipBasedUpdates,
            ], true)),
        ]),
    ]);
};
/**
  Object.entries(modelFields).forEach(([key, value]) => {
    if (typeof value === 'string' && value.trim() === "") {
      modelFields[key] = undefined;
    }
  });
 */
exports.replaceEmptyStringStatement = typescript_1.factory.createExpressionStatement(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('Object'), typescript_1.factory.createIdentifier('entries')), undefined, [typescript_1.factory.createIdentifier('modelFields')]), typescript_1.factory.createIdentifier('forEach')), undefined, [
    typescript_1.factory.createArrowFunction(undefined, undefined, [
        typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createArrayBindingPattern([
            typescript_1.factory.createBindingElement(undefined, undefined, typescript_1.factory.createIdentifier('key'), undefined),
            typescript_1.factory.createBindingElement(undefined, undefined, typescript_1.factory.createIdentifier('value'), undefined),
        ]), undefined, undefined, undefined),
    ], undefined, typescript_1.factory.createToken(typescript_1.SyntaxKind.EqualsGreaterThanToken), typescript_1.factory.createBlock([
        typescript_1.factory.createIfStatement(typescript_1.factory.createBinaryExpression(typescript_1.factory.createBinaryExpression(typescript_1.factory.createTypeOfExpression(typescript_1.factory.createIdentifier('value')), typescript_1.factory.createToken(typescript_1.SyntaxKind.EqualsEqualsEqualsToken), typescript_1.factory.createStringLiteral('string')), typescript_1.factory.createToken(typescript_1.SyntaxKind.AmpersandAmpersandToken), typescript_1.factory.createBinaryExpression(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('value'), typescript_1.factory.createIdentifier('trim')), undefined, []), typescript_1.factory.createToken(typescript_1.SyntaxKind.EqualsEqualsEqualsToken), typescript_1.factory.createStringLiteral(''))), typescript_1.factory.createBlock([
            typescript_1.factory.createExpressionStatement(typescript_1.factory.createBinaryExpression(typescript_1.factory.createElementAccessExpression(typescript_1.factory.createIdentifier('modelFields'), typescript_1.factory.createIdentifier('key')), typescript_1.factory.createToken(typescript_1.SyntaxKind.EqualsToken), typescript_1.factory.createIdentifier('undefined'))),
        ], true), undefined),
    ], true)),
]));
const buildDataStoreExpression = (dataStoreActionType, modelName, importedModelName, fieldConfigs, dataSchema, importCollection) => {
    const thisModelPrimaryKeys = dataSchema.models[modelName].primaryKeys;
    // promises.push(...statements that handle hasMany/ manyToMany/ hasOne-belongsTo relationships)
    const relationshipsPromisesAccessStatements = [];
    const hasManyRelationshipFields = [];
    const nonModelArrayFields = [];
    const savedRecordName = (0, helpers_1.lowerCaseFirst)(modelName);
    const nonModelFields = [];
    Object.entries(fieldConfigs).forEach((fieldConfig) => {
        var _a;
        const [fieldName, fieldConfigMetaData] = fieldConfig;
        const { dataType, isArray } = fieldConfigMetaData;
        if ((0, codegen_ui_1.isNonModelDataType)(dataType)) {
            if (isArray) {
                nonModelArrayFields.push(fieldName);
            }
            else {
                nonModelFields.push(fieldName);
            }
        }
        relationshipsPromisesAccessStatements.push(...(0, bidirectional_relationship_1.getBiDirectionalRelationshipStatements)({
            formActionType: dataStoreActionType,
            dataSchema,
            importCollection,
            fieldConfig,
            modelName,
            savedRecordName,
        }));
        if (((_a = fieldConfigMetaData.relationship) === null || _a === void 0 ? void 0 : _a.type) === 'HAS_MANY') {
            hasManyRelationshipFields.push(fieldName);
            if ((0, map_from_fieldConfigs_1.isManyToManyRelationship)(fieldConfigMetaData)) {
                const joinTable = dataSchema.models[fieldConfigMetaData.relationship.relatedJoinTableName];
                relationshipsPromisesAccessStatements.push(...(0, relationship_1.buildManyToManyRelationshipDataStoreStatements)(dataStoreActionType, importedModelName, fieldConfig, thisModelPrimaryKeys, joinTable, savedRecordName));
            }
            else {
                relationshipsPromisesAccessStatements.push(...(0, relationship_1.buildHasManyRelationshipDataStoreStatements)(dataStoreActionType, importedModelName, fieldConfig, thisModelPrimaryKeys, savedRecordName));
            }
        }
    });
    if (relationshipsPromisesAccessStatements.length) {
        relationshipsPromisesAccessStatements.unshift(typescript_1.factory.createVariableStatement(undefined, typescript_1.factory.createVariableDeclarationList([
            typescript_1.factory.createVariableDeclaration(typescript_1.factory.createIdentifier('promises'), undefined, undefined, typescript_1.factory.createArrayLiteralExpression([], false)),
        ], typescript_1.NodeFlags.Const)));
    }
    const propertyAssignments = [];
    hasManyRelationshipFields.forEach((field) => {
        // CPKProjects: undefined
        propertyAssignments.push(typescript_1.factory.createPropertyAssignment(typescript_1.factory.createIdentifier(field), typescript_1.factory.createIdentifier('undefined')));
    });
    propertyAssignments.push(...(0, parse_fields_1.generateParsePropertyAssignments)(nonModelArrayFields, nonModelFields));
    const modelFieldsObject = propertyAssignments.length
        ? typescript_1.factory.createObjectLiteralExpression([typescript_1.factory.createSpreadAssignment(typescript_1.factory.createIdentifier('modelFields')), ...propertyAssignments], true)
        : typescript_1.factory.createIdentifier('modelFields');
    const genericSaveStatement = relationshipsPromisesAccessStatements.length
        ? [
            typescript_1.factory.createVariableStatement(undefined, typescript_1.factory.createVariableDeclarationList([
                typescript_1.factory.createVariableDeclaration(typescript_1.factory.createIdentifier(savedRecordName), undefined, undefined, typescript_1.factory.createAwaitExpression(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('DataStore'), typescript_1.factory.createIdentifier('save')), undefined, [
                    typescript_1.factory.createNewExpression(typescript_1.factory.createIdentifier(importedModelName), undefined, [
                        modelFieldsObject,
                    ]),
                ]))),
            ], typescript_1.NodeFlags.Const)),
        ]
        : [
            typescript_1.factory.createExpressionStatement(typescript_1.factory.createAwaitExpression(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('DataStore'), typescript_1.factory.createIdentifier('save')), undefined, [
                typescript_1.factory.createNewExpression(typescript_1.factory.createIdentifier(importedModelName), undefined, [
                    modelFieldsObject,
                ]),
            ]))),
        ];
    const resolvePromisesStatement = typescript_1.factory.createExpressionStatement(typescript_1.factory.createAwaitExpression(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('Promise'), typescript_1.factory.createIdentifier('all')), undefined, [typescript_1.factory.createIdentifier('promises')])));
    const genericUpdateStatement = relationshipsPromisesAccessStatements.length
        ? [
            typescript_1.factory.createExpressionStatement(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('promises'), typescript_1.factory.createIdentifier('push')), undefined, [getRecordUpdateDataStoreCallExpression({ modelName, importedModelName, fieldConfigs })])),
            resolvePromisesStatement,
        ]
        : [
            typescript_1.factory.createExpressionStatement(typescript_1.factory.createAwaitExpression(getRecordUpdateDataStoreCallExpression({ modelName, importedModelName, fieldConfigs }))),
        ];
    if (dataStoreActionType === 'update') {
        return [...relationshipsPromisesAccessStatements, ...genericUpdateStatement];
    }
    const createStatements = [...genericSaveStatement, ...relationshipsPromisesAccessStatements];
    if (relationshipsPromisesAccessStatements.length) {
        createStatements.push(resolvePromisesStatement);
    }
    return createStatements;
};
exports.buildDataStoreExpression = buildDataStoreExpression;
/**
  example: const validationResponses = await Promise.all(
    Object.keys(validations).reduce((promises, fieldName) => {
      if (Array.isArray(modelFields[fieldName])) {
        promises.push(
          ...modelFields[fieldName].map((item) =>
            runValidationTasks(fieldName, item, getDisplayValue[fieldName]),
          ),
        );
        return promises;
      }
      promises.push(runValidationTasks(fieldName, modelFields[fieldName], getDisplayValue[fieldName]));
      return promises;
    }, []),
  );
  if (validationResponses.some((r) => r.hasError)) {
    return;
  }
*/
const onSubmitValidationRun = (shouldUseGetDisplayValue) => {
    const getDisplayValueAccess = typescript_1.factory.createElementAccessExpression(typescript_1.factory.createIdentifier(model_values_1.getDisplayValueObjectName), typescript_1.factory.createIdentifier('fieldName'));
    const runValidationTasksArgsForArray = [
        typescript_1.factory.createIdentifier('fieldName'),
        typescript_1.factory.createIdentifier('item'),
    ];
    const runValidationTasksArgs = [
        typescript_1.factory.createIdentifier('fieldName'),
        typescript_1.factory.createElementAccessExpression(typescript_1.factory.createIdentifier('modelFields'), typescript_1.factory.createIdentifier('fieldName')),
    ];
    if (shouldUseGetDisplayValue) {
        runValidationTasksArgsForArray.push(getDisplayValueAccess);
        runValidationTasksArgs.push(getDisplayValueAccess);
    }
    return [
        typescript_1.factory.createVariableStatement(undefined, typescript_1.factory.createVariableDeclarationList([
            typescript_1.factory.createVariableDeclaration(typescript_1.factory.createIdentifier('validationResponses'), undefined, undefined, typescript_1.factory.createAwaitExpression(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('Promise'), typescript_1.factory.createIdentifier('all')), undefined, [
                typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('Object'), typescript_1.factory.createIdentifier('keys')), undefined, [typescript_1.factory.createIdentifier('validations')]), typescript_1.factory.createIdentifier('reduce')), undefined, [
                    typescript_1.factory.createArrowFunction(undefined, undefined, [
                        typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createIdentifier('promises'), undefined, undefined, undefined),
                        typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createIdentifier('fieldName'), undefined, undefined, undefined),
                    ], undefined, typescript_1.factory.createToken(typescript_1.SyntaxKind.EqualsGreaterThanToken), typescript_1.factory.createBlock([
                        typescript_1.factory.createIfStatement(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('Array'), typescript_1.factory.createIdentifier('isArray')), undefined, [
                            typescript_1.factory.createElementAccessExpression(typescript_1.factory.createIdentifier('modelFields'), typescript_1.factory.createIdentifier('fieldName')),
                        ]), typescript_1.factory.createBlock([
                            typescript_1.factory.createExpressionStatement(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('promises'), typescript_1.factory.createIdentifier('push')), undefined, [
                                typescript_1.factory.createSpreadElement(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createElementAccessExpression(typescript_1.factory.createIdentifier('modelFields'), typescript_1.factory.createIdentifier('fieldName')), typescript_1.factory.createIdentifier('map')), undefined, [
                                    typescript_1.factory.createArrowFunction(undefined, undefined, [
                                        typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createIdentifier('item'), undefined, undefined, undefined),
                                    ], undefined, typescript_1.factory.createToken(typescript_1.SyntaxKind.EqualsGreaterThanToken), typescript_1.factory.createCallExpression(typescript_1.factory.createIdentifier('runValidationTasks'), undefined, runValidationTasksArgsForArray)),
                                ])),
                            ])),
                            typescript_1.factory.createReturnStatement(typescript_1.factory.createIdentifier('promises')),
                        ], true), undefined),
                        typescript_1.factory.createExpressionStatement(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('promises'), typescript_1.factory.createIdentifier('push')), undefined, [
                            typescript_1.factory.createCallExpression(typescript_1.factory.createIdentifier('runValidationTasks'), undefined, runValidationTasksArgs),
                        ])),
                        typescript_1.factory.createReturnStatement(typescript_1.factory.createIdentifier('promises')),
                    ], true)),
                    typescript_1.factory.createArrayLiteralExpression([], false),
                ]),
            ]))),
        ], typescript_1.NodeFlags.Const)),
        typescript_1.factory.createIfStatement(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('validationResponses'), typescript_1.factory.createIdentifier('some')), undefined, [
            typescript_1.factory.createArrowFunction(undefined, undefined, [
                typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createIdentifier('r'), undefined, undefined, undefined),
            ], undefined, typescript_1.factory.createToken(typescript_1.SyntaxKind.EqualsGreaterThanToken), typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('r'), typescript_1.factory.createIdentifier('hasError'))),
        ]), typescript_1.factory.createBlock([typescript_1.factory.createReturnStatement(undefined)], true), undefined),
    ];
};
exports.onSubmitValidationRun = onSubmitValidationRun;
const buildUpdateDatastoreQuery = (importedModelName, lowerCaseDataTypeName, relatedModelStatements, primaryKey) => {
    const pkQueryIdentifier = typescript_1.factory.createIdentifier(primaryKey);
    return [
        typescript_1.factory.createVariableStatement(undefined, typescript_1.factory.createVariableDeclarationList([
            typescript_1.factory.createVariableDeclaration(typescript_1.factory.createIdentifier('queryData'), undefined, undefined, typescript_1.factory.createArrowFunction([typescript_1.factory.createModifier(typescript_1.SyntaxKind.AsyncKeyword)], undefined, [], undefined, typescript_1.factory.createToken(typescript_1.SyntaxKind.EqualsGreaterThanToken), typescript_1.factory.createBlock([
                typescript_1.factory.createVariableStatement(undefined, typescript_1.factory.createVariableDeclarationList([
                    typescript_1.factory.createVariableDeclaration(typescript_1.factory.createIdentifier('record'), undefined, undefined, typescript_1.factory.createConditionalExpression(pkQueryIdentifier, typescript_1.factory.createToken(typescript_1.SyntaxKind.QuestionToken), typescript_1.factory.createAwaitExpression(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('DataStore'), typescript_1.factory.createIdentifier('query')), undefined, [typescript_1.factory.createIdentifier(importedModelName), pkQueryIdentifier])), typescript_1.factory.createToken(typescript_1.SyntaxKind.ColonToken), typescript_1.factory.createIdentifier(lowerCaseDataTypeName))),
                ], typescript_1.NodeFlags.Const)),
                typescript_1.factory.createExpressionStatement(typescript_1.factory.createCallExpression((0, helpers_1.getSetNameIdentifier)(`${lowerCaseDataTypeName}Record`), undefined, [
                    typescript_1.factory.createIdentifier('record'),
                ])),
                // Add logic to pull related relationship models off record
                ...relatedModelStatements,
            ], true))),
        ], typescript_1.NodeFlags.Const)),
        typescript_1.factory.createExpressionStatement(typescript_1.factory.createCallExpression(typescript_1.factory.createIdentifier('queryData'), undefined, [])),
    ];
};
exports.buildUpdateDatastoreQuery = buildUpdateDatastoreQuery;
//# sourceMappingURL=cta-props.js.map