"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateUpdateModelObject = exports.generateParsePropertyAssignments = exports.parseArrayValues = exports.parseValue = void 0;
/*
  Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.

  Licensed under the Apache License, Version 2.0 (the "License").
  You may not use this file except in compliance with the License.
  You may obtain a copy of the License at

      http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
const codegen_ui_1 = require("@aws-amplify/codegen-ui");
const typescript_1 = require("typescript");
/**
 * JSON.parse(s)
 */
const parseValue = (expression) => typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('JSON'), typescript_1.factory.createIdentifier('parse')), undefined, [expression]);
exports.parseValue = parseValue;
/**
 * modelFields.nonModelFieldArray.map(s => JSON.parse(item))
 */
const parseArrayValues = (accessName) => {
    return typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(accessName, typescript_1.factory.createIdentifier('map')), undefined, [
        typescript_1.factory.createArrowFunction(undefined, undefined, [
            typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createIdentifier('s'), undefined, undefined),
        ], undefined, typescript_1.factory.createToken(typescript_1.SyntaxKind.EqualsGreaterThanToken), (0, exports.parseValue)(typescript_1.factory.createIdentifier('s'))),
    ]);
};
exports.parseArrayValues = parseArrayValues;
/**
 * arrayFields = nonModelFieldArray: modelFields.nonModelFieldArray.map(s => JSON.parse(item))
 * singleFields =
 *  nonModelField: modelFields.nonModelField ? JSON.parse(modelFields.nonModelField) : modelFields.nonModelField
 */
const generateParsePropertyAssignments = (arrayFields, nonArrayFields) => {
    const parseArrayFields = arrayFields.map((field) => typescript_1.factory.createPropertyAssignment(typescript_1.factory.createIdentifier(field), (0, exports.parseArrayValues)(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('modelFields'), typescript_1.factory.createIdentifier(field)))));
    const parseFields = nonArrayFields.map((field) => typescript_1.factory.createPropertyAssignment(typescript_1.factory.createIdentifier(field), typescript_1.factory.createConditionalExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('modelFields'), typescript_1.factory.createIdentifier(field)), typescript_1.factory.createToken(typescript_1.SyntaxKind.QuestionToken), (0, exports.parseValue)(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('modelFields'), typescript_1.factory.createIdentifier(field))), typescript_1.factory.createToken(typescript_1.SyntaxKind.ColonToken), typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('modelFields'), typescript_1.factory.createIdentifier(field)))));
    return [...parseArrayFields, ...parseFields];
};
exports.generateParsePropertyAssignments = generateParsePropertyAssignments;
//
const generateUpdateModelObject = (fieldConfigs, modelFieldsObjectName) => {
    const nonModelFields = [];
    const nonModelArrayFields = [];
    Object.entries(fieldConfigs).forEach(([name, { dataType, sanitizedFieldName, isArray }]) => {
        if ((0, codegen_ui_1.isNonModelDataType)(dataType)) {
            const renderedFieldName = sanitizedFieldName || name;
            if (!isArray) {
                nonModelFields.push(renderedFieldName);
            }
            else {
                nonModelArrayFields.push(renderedFieldName);
            }
        }
    });
    const parsePropertyAssignments = (0, exports.generateParsePropertyAssignments)(nonModelArrayFields, nonModelFields);
    let updateModelObject = typescript_1.factory.createIdentifier(modelFieldsObjectName);
    if (parsePropertyAssignments.length) {
        updateModelObject = typescript_1.factory.createObjectLiteralExpression([typescript_1.factory.createSpreadAssignment(typescript_1.factory.createIdentifier(modelFieldsObjectName)), ...parsePropertyAssignments], true);
    }
    return updateModelObject;
};
exports.generateUpdateModelObject = generateUpdateModelObject;
//# sourceMappingURL=parse-fields.js.map